==> kari-0.30.tar.gz: kari-0.30/parser/staxx.c <==
/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include "tagdefs.h"
#include "utils.h"
#include "staxx.h"
#include "neve.h"


/* This module deals with push and pop operations for the variouus
 * stacks needed to keep track of the state of font size/colour,
 * header levels, paragraph alignments, and list indention and
 * numbering levels. This allows nested <FONT> and <HEADER> tags even
 * thought the latter is incorrect HTML. It also allows correct
 * indentation of lists and correct numbering when dealing with nested
 * lists.
 */

void push_align (neve_alignment alignment, int tag)
{
  if (!align_stack)
    {
      debug (debug_parser, "staxx: Creating alignment stack.");
      align_stack = xmalloc (sizeof(*align_stack));
      align_stack->previous = NULL;
      align_stack->next = NULL; /* May not need this ... */
    }
  else
    {
      debug (debug_parser, "staxx: Extending alignment stack.");
      align_stack->next = xmalloc (sizeof(*align_stack));
      align_stack->next->previous = align_stack;
      align_stack = align_stack->next;
    }
  align_stack->alignment = alignment;
  align_stack->tag = tag;
  debug (debug_parser, "staxx: Pushed alignment %d onto align stack.", align_stack->alignment);
  align_stack->next = NULL;  /* YARK, do we need this? */
}



void pop_align (physical_attribute_types *pattribs, int tag)
{
  align_node *temp;

  if (!align_stack)
    {
      debug (debug_parser, "staxx: No align_stack available, returning.");
      pattribs->alignment = left; /* DEFAULT */
      return;
    }
  else
    {
      if (tag == DIV)
	{
	  debug (debug_parser, "staxx: popping for a DIV element");
	  if (align_stack->tag == PARAGRAPH)
	    debug (debug_parser, "staxx: top is P");
	  else if (align_stack->tag == DIV)
	    debug (debug_parser, "staxx: top is DIV");
	  else
	    debug (debug_parser, "staxx: top is %d", align_stack->tag);

	  if (align_stack->tag == PARAGRAPH)
	    {
	      debug ( debug_parser, "staxx: remove the P info first ...");
	      pattribs->alignment = align_stack->alignment;
	      debug (debug_parser, "staxx: Popped alignment %d off p_stack.", align_stack->alignment);
	      temp = align_stack;
	      align_stack = align_stack->previous;
	      free (temp);
	      if (align_stack)
		align_stack->next = NULL;
	    }
	  debug ( debug_parser, "staxx: popping the DIV info ...");
	  pattribs->alignment = align_stack->alignment;
	  debug (debug_parser, "staxx: Popped alignment %d off p_stack.", align_stack->alignment);
	  temp = align_stack;
	  align_stack = align_stack->previous;
	  free (temp);
	  if (align_stack)
	    align_stack->next = NULL;
	}
      if (tag == PARAGRAPH && align_stack->tag == PARAGRAPH)
	{
	  pattribs->alignment = align_stack->alignment;
	  debug (debug_parser, "staxx: Popped alignment %d off p_stack.", align_stack->alignment);
	  temp = align_stack;
	  align_stack = align_stack->previous;
	  free (temp);
	  if (align_stack)
	    align_stack->next = NULL;
	}
    }
}



void push_font (colour_type colour,int size)
{
  if (!font_stack)
    {
      debug (debug_parser, "staxx: Creating font_stack.");
      font_stack = xmalloc (sizeof(*font_stack));
      font_stack->previous = NULL;
      font_stack->next = NULL; /* YARK, do we need this? */
    }
  else
    {
      debug (debug_parser, "staxx: Extending font stack.");
      font_stack->next = xmalloc(sizeof(*font_stack));
      font_stack->next->previous = font_stack;
      font_stack = font_stack->next;
    }
  font_stack->colour = colour;
  font_stack->size = size;
  debug (debug_parser, "staxx: Pushed font size=%d, colour=%d.%d.%d onto stack.", size, colour.red, colour.green, colour.blue);
  font_stack->next = NULL;
  font_stack->previous = NULL; /* YARK, do we need this */
}				 



void pop_font (physical_attribute_types *pattribs)
{
  font_node *temp;
  
  if (!font_stack)
    {
      debug (debug_parser, "staxx: No font_stack available, returning.");
      return;
    }
  else 
    {
      pattribs->draw_colour = font_stack->colour;
      debug (debug_parser, "staxx: Popped colour %d.%d.%d off stack.", 
	     pattribs->draw_colour.red,
	     pattribs->draw_colour.green,
	     pattribs->draw_colour.blue); 
      pattribs->font_level = font_stack->size;
      debug (debug_parser, "staxx: Popped size %d off stack.",
	     pattribs->font_level);
    }
  
  temp = font_stack;
  font_stack = font_stack->previous;
  free (temp);
  if (font_stack)
    font_stack->next = NULL;
}



void push_header (neve_alignment alignment, int font_level, boolean bold_status)
{
  if (!header_stack)
    {
      debug (debug_parser, "staxx: Creating header_stack.");
      header_stack = xmalloc (sizeof(*header_stack));
      header_stack->previous = NULL;
    }
  else
    {
      debug (debug_parser, "staxx: Extending header_stack.");
      header_stack->next = xmalloc(sizeof(*header_stack));
      header_stack->next->previous = header_stack;
      header_stack = header_stack->next;
    }
  header_stack->alignment = alignment;
  header_stack->level = font_level;
  header_stack->bold = bold_status;
  debug(debug_parser, "staxx: Pushed align=%d, level=%d onto header_stack.",
	header_stack->alignment,
	header_stack->level);
  header_stack->next = NULL;
}



void pop_header (physical_attribute_types *current_pattribs)
{
  header_node *temp;

  if (!header_stack)
    {
      debug (debug_parser, "staxx: No header_stack available, returning.");
      return; /* YARK, do we need to supply any default values before returning? */
    }
  else
    {
      current_pattribs->font_level = header_stack->level;
      current_pattribs->alignment = header_stack->alignment;
      current_pattribs->strong = header_stack->bold;
      debug (debug_parser, "staxx: Popped align=%d, level=%d off header_stack.",
	     current_pattribs->alignment,
	     current_pattribs->font_level);
      temp = header_stack;
      header_stack = header_stack->previous;
      free (temp);
      if (header_stack)
	header_stack->next = NULL;
    }
}



void push_list (logical_attribute_types *lattribs)
{
  if (!list_stack)
    {
      debug (debug_parser, "staxx: Creating list_stack.");
      list_stack = xmalloc (sizeof(*list_stack));
      list_stack->previous = NULL;
    }
  else
    {
      debug (debug_parser, "staxx: Extending list_stack.");
      list_stack->next = xmalloc(sizeof(*list_stack));
      list_stack->next->previous = list_stack;
      list_stack = list_stack->next;
    }
  list_stack->ol = lattribs->ol_list_element;
  list_stack->ul = lattribs->ul_list_element;
  list_stack->dl = lattribs->dl_list_element;
  list_stack->indent_level = lattribs->list_indent_level;
  list_stack->count = lattribs->item_number;
  debug (debug_parser, "staxx: Pushed element onto stack ...");
  if (list_stack->ol)
    debug (debug_parser, "staxx: ordered list");
  if (list_stack->ul)
    debug (debug_parser, "staxx: unordered list");
  if (list_stack->dl)
    debug (debug_parser, "staxx: definition list");
  debug (debug_parser, "staxx: level=%d, count=%d",
	 list_stack->indent_level,
	 list_stack->count);
  list_stack->next = NULL;
}



void pop_list (logical_attribute_types *lattribs)
{
  list_node *temp;
  if (!list_stack)
    {
      debug(debug_parser, "staxx: No list_stack available, returning.");
      return;
    }
  else
    {
      lattribs->list_indent_level = list_stack->indent_level;
      lattribs->item_number = list_stack->count;
      lattribs->ol_list_element = list_stack->ol;
      lattribs->ul_list_element = list_stack->ul;
      lattribs->dl_list_element = list_stack->dl;
      debug (debug_parser, "staxx: Popped element from stack ...");
      if (list_stack->ol)
	debug (debug_parser, "staxx: ordered list");
      if (list_stack->ul)
	debug (debug_parser, "staxx: unordered list");
      if (list_stack->dl)
	debug (debug_parser, "staxx: definition list");
      debug (debug_parser, "staxx: level=%d, count=%d",
	     list_stack->indent_level,
	     list_stack->count);
      temp = list_stack;
      list_stack = list_stack->previous;
      free (temp);
      
      if (list_stack)
	list_stack->next = NULL;
    }
}

==> kari-0.31.tar.gz: kari-0.31/parser/staxx.c <==
/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include "tagdefs.h"
#include "utils.h"
#include "staxx.h"
#include "neve.h"


/* This module deals with push and pop operations for the variouus
 * stacks needed to keep track of the state of font size/colour,
 * header levels, paragraph alignments, and list indention and
 * numbering levels. This allows nested <FONT> and <HEADER> tags even
 * thought the latter is incorrect HTML. It also allows correct
 * indentation of lists and correct numbering when dealing with nested
 * lists.
 */

void push_align (neve_alignment alignment, int tag)
{
  if (!align_stack)
    {
      debug (debug_parser, "staxx: Creating alignment stack.");
      align_stack = xmalloc (sizeof(*align_stack));
      align_stack->previous = NULL;
      align_stack->next = NULL; /* May not need this ... */
    }
  else
    {
      debug (debug_parser, "staxx: Extending alignment stack.");
      align_stack->next = xmalloc (sizeof(*align_stack));
      align_stack->next->previous = align_stack;
      align_stack = align_stack->next;
    }
  align_stack->alignment = alignment;
  align_stack->tag = tag;
  debug (debug_parser, "staxx: Pushed alignment %d onto align stack.", align_stack->alignment);
  align_stack->next = NULL;  /* YARK, do we need this? */
}



void pop_align (physical_attribute_types *pattribs, int tag)
{
  align_node *temp;

  if (!align_stack)
    {
      debug (debug_parser, "staxx: No align_stack available, returning.");
      pattribs->x_alignment = left; /* DEFAULT */
      return;
    }
  else
    {
      if (tag == DIV)
	{
	  debug (debug_parser, "staxx: popping for a DIV element");
	  if (align_stack->tag == PARAGRAPH)
	    debug (debug_parser, "staxx: top is P");
	  else if (align_stack->tag == DIV)
	    debug (debug_parser, "staxx: top is DIV");
	  else
	    debug (debug_parser, "staxx: top is %d", align_stack->tag);

	  if (align_stack->tag == PARAGRAPH)
	    {
	      debug ( debug_parser, "staxx: remove the P info first ...");
	      pattribs->x_alignment = align_stack->alignment;
	      debug (debug_parser, "staxx: Popped alignment %d off p_stack.", align_stack->alignment);
	      temp = align_stack;
	      align_stack = align_stack->previous;
	      free (temp);
	      if (align_stack)
		align_stack->next = NULL;
	    }
	  debug ( debug_parser, "staxx: popping the DIV info ...");
	  pattribs->x_alignment = align_stack->alignment;
	  debug (debug_parser, "staxx: Popped alignment %d off p_stack.", align_stack->alignment);
	  temp = align_stack;
	  align_stack = align_stack->previous;
	  free (temp);
	  if (align_stack)
	    align_stack->next = NULL;
	}
      if (tag == PARAGRAPH && align_stack->tag == PARAGRAPH)
	{
	  pattribs->x_alignment = align_stack->alignment;
	  debug (debug_parser, "staxx: Popped alignment %d off p_stack.", align_stack->alignment);
	  temp = align_stack;
	  align_stack = align_stack->previous;
	  free (temp);
	  if (align_stack)
	    align_stack->next = NULL;
	}
    }
}



void push_font (colour_type colour,int size)
{
  if (!font_stack)
    {
      debug (debug_parser, "staxx: Creating font_stack.");
      font_stack = xmalloc (sizeof(*font_stack));
      font_stack->previous = NULL;
      font_stack->next = NULL; /* YARK, do we need this? */
    }
  else
    {
      debug (debug_parser, "staxx: Extending font stack.");
      font_stack->next = xmalloc(sizeof(*font_stack));
      font_stack->next->previous = font_stack;
      font_stack = font_stack->next;
    }
  font_stack->colour = colour;
  font_stack->size = size;
  debug (debug_parser, "staxx: Pushed font size=%d, colour=%d.%d.%d onto stack.", size, colour.red, colour.green, colour.blue);
  font_stack->next = NULL;
  font_stack->previous = NULL; /* YARK, do we need this */
}				 



void pop_font (physical_attribute_types *pattribs)
{
  font_node *temp;
  
  if (!font_stack)
    {
      debug (debug_parser, "staxx: No font_stack available, returning.");
      return;
    }
  else 
    {
      pattribs->draw_colour = font_stack->colour;
      debug (debug_parser, "staxx: Popped colour %d.%d.%d off stack.", 
	     pattribs->draw_colour.red,
	     pattribs->draw_colour.green,
	     pattribs->draw_colour.blue); 
      pattribs->font_level = font_stack->size;
      debug (debug_parser, "staxx: Popped size %d off stack.",
	     pattribs->font_level);
    }
  
  temp = font_stack;
  font_stack = font_stack->previous;
  free (temp);
  if (font_stack)
    font_stack->next = NULL;
}



void push_header (neve_alignment alignment, int font_level, boolean bold_status)
{
  if (!header_stack)
    {
      debug (debug_parser, "staxx: Creating header_stack.");
      header_stack = xmalloc (sizeof(*header_stack));
      header_stack->previous = NULL;
    }
  else
    {
      debug (debug_parser, "staxx: Extending header_stack.");
      header_stack->next = xmalloc(sizeof(*header_stack));
      header_stack->next->previous = header_stack;
      header_stack = header_stack->next;
    }
  header_stack->alignment = alignment;
  header_stack->level = font_level;
  header_stack->bold = bold_status;
  debug(debug_parser, "staxx: Pushed align=%d, level=%d onto header_stack.",
	header_stack->alignment,
	header_stack->level);
  header_stack->next = NULL;
}



void pop_header (physical_attribute_types *current_pattribs)
{
  header_node *temp;

  if (!header_stack)
    {
      debug (debug_parser, "staxx: No header_stack available, returning.");
      return; /* YARK, do we need to supply any default values before returning? */
    }
  else
    {
      current_pattribs->font_level = header_stack->level;
      current_pattribs->x_alignment = header_stack->alignment;
      current_pattribs->strong = header_stack->bold;
      debug (debug_parser, "staxx: Popped align=%d, level=%d off header_stack.",
	     current_pattribs->x_alignment,
	     current_pattribs->font_level);
      temp = header_stack;
      header_stack = header_stack->previous;
      free (temp);
      if (header_stack)
	header_stack->next = NULL;
    }
}



void push_list (logical_attribute_types *lattribs)
{
  if (!list_stack)
    {
      debug (debug_parser, "staxx: Creating list_stack.");
      list_stack = xmalloc (sizeof(*list_stack));
      list_stack->previous = NULL;
    }
  else
    {
      debug (debug_parser, "staxx: Extending list_stack.");
      list_stack->next = xmalloc(sizeof(*list_stack));
      list_stack->next->previous = list_stack;
      list_stack = list_stack->next;
    }
  list_stack->ol = lattribs->ol_list_element;
  list_stack->ul = lattribs->ul_list_element;
  list_stack->dl = lattribs->dl_list_element;
  list_stack->indent_level = lattribs->list_indent_level;
  list_stack->count = lattribs->item_number;
  debug (debug_parser, "staxx: Pushed element onto stack ...");
  if (list_stack->ol)
    debug (debug_parser, "staxx: ordered list");
  if (list_stack->ul)
    debug (debug_parser, "staxx: unordered list");
  if (list_stack->dl)
    debug (debug_parser, "staxx: definition list");
  debug (debug_parser, "staxx: level=%d, count=%d",
	 list_stack->indent_level,
	 list_stack->count);
  list_stack->next = NULL;
}



void pop_list (logical_attribute_types *lattribs)
{
  list_node *temp;
  if (!list_stack)
    {
      debug(debug_parser, "staxx: No list_stack available, returning.");
      return;
    }
  else
    {
      lattribs->list_indent_level = list_stack->indent_level;
      lattribs->item_number = list_stack->count;
      lattribs->ol_list_element = list_stack->ol;
      lattribs->ul_list_element = list_stack->ul;
      lattribs->dl_list_element = list_stack->dl;
      debug (debug_parser, "staxx: Popped element from stack ...");
      if (list_stack->ol)
	debug (debug_parser, "staxx: ordered list");
      if (list_stack->ul)
	debug (debug_parser, "staxx: unordered list");
      if (list_stack->dl)
	debug (debug_parser, "staxx: definition list");
      debug (debug_parser, "staxx: level=%d, count=%d",
	     list_stack->indent_level,
	     list_stack->count);
      temp = list_stack;
      list_stack = list_stack->previous;
      free (temp);
      
      if (list_stack)
	list_stack->next = NULL;
    }
}
